; ID: 3141
; Author: RemiD
; Date: 2014-08-26 12:25:04
; Title: basic collisions example
; Description: turningmoving colliders (ellipsoid) -> static collidables (mesh) | turningmoving colliders (ellipsoid) <-> turningmoving colliders (ellipsoid)

;Here is an example of how to use the collision system and how to manage the turningmoving colliders and the static collidables
;collidables = static meshes
;colliders = turningmoving ellipsoids
;turningmoving colliders (ellipsoid) -> static collidables (mesh) | turningmoving colliders (ellipsoid) <-> turningmoving colliders (ellipsoid)

Graphics3D(640,480,32,2)
SetBuffer(BackBuffer())

SeedRnd(MilliSecs())

Camera = CreateCamera()
CameraRange(Camera,0.1,100)
PositionEntity(Camera,0,5,-5)
RotateEntity(Camera,45,0,0)

;create position rotate the statics collidables (meshes)
Global StaticsCount%
Dim Static(10)

For i% = 1 To 10
 StaticsCount = StaticsCount + 1
 Id% = StaticsCount
 Static(Id) = CreateCube()
 ScaleMesh(Static(Id),0.5/2,0.5/2,0.5/2)
 PositionMesh(Static(Id),0,0.5/2,0)
 .LineChooseStaticPosition
 PositionEntity(Static(Id),Rnd(-5,5),0,Rnd(-5,5))
 ;is there another static at this position ?
 For OId% = 1 To StaticsCount
  If(OId <> Id)
   If(EntityDistance(Static(Id),Static(OId)) < 0.36+0.36)
    Goto LineChooseStaticPosition
   EndIf
  EndIf
 Next
Next

;create position rotate the turningmovings colliders (ellipsoids)
Global TurningMovingsCount%
Dim TurningMoving(10)

For i% = 1 To 2
 TurningMovingsCount = TurningMovingsCount + 1
 Id% = TurningMovingsCount
 TurningMoving(Id) = CreateSphere(16)
 ScaleMesh(TurningMoving(Id),0.5/2,0.5/2,0.5/2)
 DebugTurningMovingDirection = CreateCube()
 ScaleMesh(DebugTurningMovingDirection,0.01/2,0.01/2,1.0/2)
 PositionMesh(DebugTurningMovingDirection,0,0,1.0/2)
 PositionEntity(DebugTurningMovingDirection,EntityX(TurningMoving(Id),True),EntityY(TurningMoving(Id),True),EntityZ(TurningMoving(Id),True))
 RotateEntity(DebugTurningMovingDirection,EntityPitch(TurningMoving(Id),True),EntityYaw(TurningMoving(Id),True),EntityRoll(TurningMoving(Id),True))
 EntityParent(DebugTurningMovingDirection,TurningMoving(Id),True)
 .LineChooseTurningMovingPosition
 PositionEntity(TurningMoving(Id),Rnd(-5,5),0.25,Rnd(-5,5))
 ;is there a static at this position ?
 For OId% = 1 To StaticsCount
  If(EntityDistance(TurningMoving(Id),Static(OId)) < 0.25+0.36)
   Goto LineChooseTurningMovingPosition
  EndIf
 Next
 ;is there another turningmoving at this position ?
 For OId% = 1 To StaticsCount
  If(OId <> Id)
   If(EntityDistance(Static(Id),Static(OId)) < 0.25+0.25)
    Goto LineChooseTurningMovingPosition
   EndIf
  EndIf
 Next
 RotateEntity(TurningMoving(Id),0,Rnd(-180,180),0)
Next

;put each static collidable in a collision group
For Id% = 1 To StaticsCount
 EntityType(Static(Id),1) ;group 1
Next

;put each turningmoving collider in a collision group
For Id% = 1 To TurningMovingsCount
 EntityRadius(TurningMoving(Id),0.25)
 EntityType(TurningMoving(Id),2) ;group 2
Next

;configure the collision detection and response between the different collisions groups
Collisions(2,1,2,1) ;collision group 2 -> group 1, ellipsoid to mesh, stop
Collisions(2,2,1,1) ;collision group 2 -> group 2, ellipsoid to ellipsoid, stop

While(KeyDown(1)=0)

 ;turn move turningmoving 1
 If(KeyDown(30)=1)
  TurnEntity(TurningMoving(1),0,1.5,0)
 ElseIf(KeyDown(32)=1)
  TurnEntity(TurningMoving(1),0,-1.5,0)
 EndIf
 If(KeyDown(17)=1)
  MoveEntity(TurningMoving(1),0,0,0.05)
 ElseIf(KeyDown(31)=1)
  MoveEntity(TurningMoving(1),0,0,-0.05)
 EndIf
 
 ;turn move turningmoving 2
 If(KeyDown(203)=1)
  TurnEntity(TurningMoving(2),0,1.5,0)
 ElseIf(KeyDown(205)=1)
  TurnEntity(TurningMoving(2),0,-1.5,0)
 EndIf
 If(KeyDown(200)=1)
  MoveEntity(TurningMoving(2),0,0,0.05)
 ElseIf(KeyDown(208)=1)
  MoveEntity(TurningMoving(2),0,0,-0.05)
 EndIf
 
 ;detect collisions update colliders depending on the "response"
 UpdateWorld()

 For Id% = 1 To StaticsCount
  EntityColor(Static(Id),255,255,255)
 Next
 For Id% = 1 To TurningMovingsCount
  EntityColor(TurningMoving(Id),255,255,255)
 Next

 ;for each turningmoving collider
 For Id% = 1 To TurningMovingsCount
  ;check if a collision happened between this turningmoving collider and a static collidable or another turningmoving collider
  CC% = CountCollisions(TurningMoving(Id))
  ;if yes
  If(CC > 0)
   ;retrieve the collided with CollisionEntity(Collider,1) ;note : in most cases there is only one collision but if CountCollisions(Collider) is superior to 1, you can analyze the others collisions
   ;retrieve the collision point with CollisionX(Collider,1) CollisionY(Collider,1) CollisionZ(Collider,1) ;same note as above
   ;retrieve the collision normal with CollisionNX(Collider,1) CollisionNY(Collider,1) CollisionNZ(Collider,1) ;same note as above
   ;update the entities as you want...
   EntityColor(TurningMoving(Id),255,000,000) ;turningmoving collider (ellipsoid)
   EntityColor(CollisionEntity(TurningMoving(Id),1),125,000,000) ;static collidable (mesh) or another turningmoving collider (ellipsoid)
  EndIf
 Next

 RenderWorld()

 Flip(1)

Wend

End()

