' ID: 2965
' Author: Madk
' Date: 2012-07-30 13:05:26
' Title: Compute fast ceil/floor of log base 2
' Description: Considerably faster alternatives to using floats or doubles

' 	--+-----------------------------------------------------------------------------------------+--
'	  | This code was originally written by Sophie Kirschner (meapineapple@gmail.com) and it is |  
' 	  | released as public domain. Please do not interpret that as liberty to claim credit that |  
' 	  | is not yours, or to sell this code when it could otherwise be obtained for free because |  
'	  |                    that would be a really shitty thing of you to do.                    |
' 	--+-----------------------------------------------------------------------------------------+--

SuperStrict


' Example code
Rem
Local ms%,nms%
Local i%,x!,y%
Const log2!=0.69314718055994529

' eat some cycles before going on with the important part
For i%=0 Until 50000
	x=Ceil(Log(i)/log2)
Next

' test using doubles
ms=MilliSecs()
For i%=0 Until 5000000
	x=Ceil(Log(i)/log2)
Next
nms=MilliSecs()-ms
Print "Ceil(Log2(x)) using doubles: "+nms+" ms"

' test using clog2()
ms=MilliSecs()
For i%=0 Until 5000000
	y=clog2(i)
Next
nms=MilliSecs()-ms
Print "Ceil(Log2(x)) using clog2(): "+nms+" ms"
EndRem 


Private
Global log2_array%[]=[$80000000,$40000000,$20000000,$10000000,$08000000,$04000000,$02000000,$01000000, ..
			   $00800000,$00400000,$00200000,$00100000,$00080000,$00040000,$00020000,$00010000, ..
			   $00008000,$00004000,$00002000,$00001000,$00000800,$00000400,$00000200,$00000100, ..
			   $00000080,$00000040,$00000020,$00000010,$00000008,$00000004,$00000002,$00000001]

Public
Rem
bbdoc: Returns Ceil( Log2( x ) ) when x is a positive integer
about: Much faster than using floating point or double functions. Good for when you don't need to know the exact value.
EndRem
Function clog2%(x%)
	For Local y%=0 Until 32
		If (x & log2_array[y]) Then Return (31-y)+(x<>log2_array[y])
	Next
	Return -1 ' this means that x was not positive
End Function
Rem
bbdoc: Returns Floor( Log2( x ) ) when x is a positive integer
about: Much faster than using floating point or double functions. Good for when you don't need to know the exact value.
EndRem
Function flog2%(x%)
	For Local y%=0 Until 32
		If (x & log2_array[y]) Then Return (31-y)
	Next
	Return -1 ' this means that x was not positive
End Function

