' ID: 2734
' Author: Madk
' Date: 2010-06-25 16:48:00
' Title: Parsing functions
' Description: Parse a text file full of commands with ease!

'This code is public domain. It is originally written by Madk.
'http://blitzbasic.com/codearcs/codearcs.php?code=2734

	SuperStrict
	
	Import brl.retro
	Import brl.system
	
	
	Private
	
	Global paramchars$="~~~q` /\.!@#$%^&*()[]{}|:'_-+1234567890qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM"
	Global paramquantity@=10

	Function ResetParams()
		numparams=0
		For Local reset%=0 To paramquantity-1
			param[reset]=""
		Next
	End Function
	Function CleanParams()
		For Local clean%=0 To numparams-1
			param[clean]=Replace(param[clean],Chr(9)," ") 'tab
			param[clean]=Replace(param[clean],Chr(34)," ")'return
			param[clean]=Trim(param[clean])
		Next
	End Function
	

	Public
	
	Global param$[paramquantity],numparams@
	Global comment$="//"
	
	Function Parse$(line$,disp$="")
		line=Trim(Replace(line,Chr(9)," "))
		Local name$=""
		If Left(Trim(line),Len(comment))=comment Then CleanParams;Return ""
		ResetParams()
		Local pos@@=0,onparam%=-1
		Repeat
			pos:+1
			Local char$=Mid(line,pos,1)
			If char=" " And onparam=-1 Then onparam=0;numparams=1;char=""
			If char="," Then onparam:+1;numparams:+1;char=""
			If onparam>paramquantity-1 Then Exit
			If Instr(line,comment)=pos Then Exit
			If Instr(paramchars,char) Then
				If onparam=-1
					If Not(char=" " Or char="~q") Then name:+char
				Else
					If char="~q" Then 'if it's in quotes, then anything goes in a param.
						Repeat
							pos:+1
							char=Mid(line,pos,1)
							If char="~q" Then Exit
							param[onparam]:+char
						Until pos>Len(line)
						If pos>Len(line) Then CleanParams;Notify "Improperly teminated string "+disp+" ( "+param[onparam]+" )",1;Return name
					Else
						param[onparam]:+char
					EndIf
				EndIf
			EndIf
		Until pos>Len(line)
		CleanParams
		If numparams=1
			If param[numparams-1]="" numparams:-1
		EndIf
		Return Trim(name)
	End Function





	Function LeftIs@(str$,sub$)
		If Len(sub)>Len(str) Return 0
		If Instr(str,sub)=0 Return 1
		Return 0
	End Function
	
	
	Function Hex2Dec:Long(hexstring$,msg$="")
		hexstring=Upper(hexstring)
		If Left(hexstring,1)="$" Then hexstring=Right(hexstring,Len(hexstring)-1)
		If Not Instr("0123456789ABCDEF",Left(hexstring,1))
			Notify "Could not translate unrecognized hexadecimal string "+msg,1
			Return 0
		EndIf
		Local value:Long
		For Local along@=1 To Len(hexstring)
			Local char@
			Select Mid(hexstring,along,1)
				Case 0,1,2,3,4,5,6,7,8,9 char=Byte(Mid(hexstring,along,1))
				Case "A" char=10
				Case "B" char=11
				Case "C" char=12
				Case "D" char=13
				Case "E" char=14
				Case "F" char=15
				Default Notify "Invalid character in hexadecimal string "+msg,1
			End Select
			value:+(char Shl ((Len(hexstring)-(along))*4))
		Next
		Return value
	End Function
	
	Function Bin2Dec:Long(binstring$,msg$="")
		If Left(binstring,1)="%" Then binstring=Right(binstring,Len(binstring)-1)
		If Not Instr("01",Left(binstring,1))
			Notify "Could not translate unrecognized binary string "+msg,1
			Return 0
		EndIf
		Local value:Long
		For Local along@=1 To Len(binstring)
			If Mid(binstring,along,1)<>"0" And Mid(binstring,along,1)<>"1" Notify "Invalid character in binary string "+msg,1
			Local char@=Byte(Mid(binstring,along,1))
			value:+(char Shl ((Len(binstring)-(along))))
		Next
		Return value
	End Function
	
	Function Oct2Dec:Long(octstring$,msg$="")
		If Left(octstring,1)="*" Then octstring=Right(octstring,Len(octstring)-1)
		If Not Instr("01234567",Left(octstring,1))
			Notify "Could not translate unrecognized base 8 string "+msg,1
			Return 0
		EndIf
		Local value:Long
		For Local along@=1 To Len(octstring)
			If Not Instr("01234567",Mid(octstring,along,1)) Notify "Invalid character in base 8 string "+msg,1
			Local char@=Byte(Mid(octstring,along,1))
			value:+(char Shl ((Len(octstring)-(along))*3))
		Next
		Return value
	End Function

	
	
	Function ReadValue:String(line$,txt$,defval$)
		If Lower(Left(line,Len(txt)))=Lower(txt) And Instr(line,"=")>0 Then
			Local parse$="",toggle%=0
			For Local xx%=Len(txt) To Len(line)
				If Instr(line,comment)=xx Then toggle=-1
				If toggle=1 Then parse:+Mid(line,xx,1)
				If Mid(line,xx,1)="=" And toggle=0 Then toggle=1
			Next
			Return Replace(Trim(parse),"~q","")
		Else
			Return defval
		EndIf
	End Function

