' ID: 3217
' Author: Spencer
' Date: 2015-08-01 03:09:46
' Title: SendXMLHttpRequest
' Description: Send Synchronous GET or POST Requests

Const HTTP_METHOD_GET:Int = 1
Const HTTP_METHOD_POST:Int = 2

Function SendXMLHttpRequest:String( Host:String, Path:String, Port:Int=80, RequestMethod:Int=HTTP_METHOD_GET, ArgsData:String[])

	Local TcpSocket:TSocket 
	Local Stream:TSocketStream
	Local ResponseText:String = ""
	Local ArgsString:String = "";
	Local ContentLength:Int = 0
	Local RequestHeaderStub:String = ""
	Local ResponseHeader:String
    Local i:Int = 0
	
	TcpSocket = CreateTCPSocket()
	ConnectSocket(TcpSocket,HostIp(Host),Port)
	Stream = CreateSocketStream(TcpSocket,True)

	Select RequestMethod
	
		Case HTTP_METHOD_GET
			
			RequestHeaderStub = "GET " + Path + "?"
			For i = 0 To ArgsData.Length-1 Step 2
				RequestHeaderStub :+ "&" + EscapeString(ArgsData[i]) + "=" + EscapeString(ArgsData[i+1])
			Next
			RequestHeaderStub :+ " HTTP/1.0"
			WriteLine(Stream,RequestHeaderStub)
			WriteLine(Stream,"HOST: " + Host)
			WriteLine(Stream,"")
			
		Case HTTP_METHOD_POST

			For i = 0 To ArgsData.Length-1 Step 2
				ArgsString = ArgsString + EscapeString(ArgsData[i]) + "=" + EscapeString(ArgsData[i+1]) + "&"
			Next
			DebugStop
			
			WriteLine(Stream,"POST " + Path + " HTTP/1.0")
			WriteLine(Stream,"Accept: */*")
			WriteLine(Stream,"Host: "+ Host)
			WriteLine(Stream,"Content-type: application/x-www-form-urlencoded")
			WriteLine(Stream,"User-agent: blitzconn") 
			WriteLine(Stream,"Content-length: " + ArgsString.Length)
			WriteLine(Stream,"Pragma: no-cache")
			WriteLine(Stream,"Connection: keep-alive")
			WriteLine(Stream,"")
			WriteLine(Stream,ArgsString)
			WriteLine(Stream,"")
			
	End Select	
		
	FlushStream(Stream)
	
	ResponseHeader = ReadLine(Stream)
	
	While ResponseHeader <> ""
	
		If Instr(ResponseHeader, "Content-Length:") > 0 Then
	
			ResponseHeader = Replace(ResponseHeader, "Content-Length:", "")
			ResponseHeader = Trim(ResponseHeader)
			ContentLength = Int(ResponseHeader)
			
			For Local b:Int = 0 To ContentLength-1
				ResponseText :+ Chr(ReadByte(Stream))	
			Next
			Exit
		Else
			ResponseHeader = ReadLine(Stream)
		EndIf
	Wend
	
	CloseStream(Stream)
	
	Return ResponseText
	
End Function


Function EscapeString:String(Value:String)

	Local EscapedValue:String = ""
	Local CharIndex:Int = 0
	
	For CharIndex = 1 To Value.Length
		EscapedValue :+ GetEscapeChar(Mid(Value,CharIndex,1))
	Next
	
	Return EscapedValue
	
End Function


Function GetEscapeChar:String(Char:String)
	Select Char
		Case "~t" ; Return "%09"
		Case "~n" ; Return "%0A"
		Case "~r" ; Return "%0D"
		Case " "  ; Return "%20"
		Case "!"  ; Return "%21"
		Case "~q" ; Return "%22"
		Case "#"  ; Return "%23"
		Case "$"  ; Return "%24"
		Case "%"  ; Return "%25"
		Case "&"  ; Return "%26"
		Case "'"  ; Return "%27"
		Case "("  ; Return "%28"
		Case ")"  ; Return "%29"
		Case "*"  ; Return "%2A"
		Case "+"  ; Return "%2B"
		Case ","  ; Return "%2C"
		Case "-"  ; Return "%2D"
		Case "."  ; Return "%2E"
		Case "/"  ; Return "%2F"
		Rem 
		Case "0"  ; Return "%30"
		Case "1"  ; Return "%31"
		Case "2"  ; Return "%32"
		Case "3"  ; Return "%33"
		Case "4"  ; Return "%34"
		Case "5"  ; Return "%35"
		Case "6"  ; Return "%36"
		Case "7"  ; Return "%37"
		Case "8"  ; Return "%38"
		Case "9"  ; Return "%39"
		End Rem
		Case ":"  ; Return "%3A"
		Case ";"  ; Return "%3B"
		Case "<"  ; Return "%3C"
		Case "="  ; Return "%3D"
		Case ">"  ; Return "%3E"
		Case "?"  ; Return "%3F"
		Case "@"  ; Return "%40"
		Rem 
		Case "A"  ; Return "%41"
		Case "B"  ; Return "%42"
		Case "C"  ; Return "%43"
		Case "D"  ; Return "%44"
		Case "E"  ; Return "%45"
		Case "F"  ; Return "%46"
		Case "G"  ; Return "%47"
		Case "H"  ; Return "%48"
		Case "I"  ; Return "%49"
		Case "J"  ; Return "%4A"
		Case "K"  ; Return "%4B"
		Case "L"  ; Return "%4C"
		Case "M"  ; Return "%4D"
		Case "N"  ; Return "%4E"
		Case "O"  ; Return "%4F"
		Case "P"  ; Return "%50"
		Case "Q"  ; Return "%51"
		Case "R"  ; Return "%52"
		Case "S"  ; Return "%53"
		Case "T"  ; Return "%54"
		Case "U"  ; Return "%55"
		Case "V"  ; Return "%56"
		Case "W"  ; Return "%57"
		Case "X"  ; Return "%58"
		Case "Y"  ; Return "%59"
		Case "Z"  ; Return "%5A"
		End Rem
		Case "["  ; Return "%5B"
		Case "\"  ; Return "%5C"
		Case "]"  ; Return "%5D"
		Case "^"  ; Return "%5E"
		Case "_"  ; Return "%5F"
		Case "`"  ; Return "%60"
		Rem 
		Case "a"  ; Return "%61"
		Case "b"  ; Return "%62"
		Case "c"  ; Return "%63"
		Case "d"  ; Return "%64"
		Case "e"  ; Return "%65"
		Case "f"  ; Return "%66"
		Case "g"  ; Return "%67"
		Case "h"  ; Return "%68"
		Case "i"  ; Return "%69"
		Case "j"  ; Return "%6A"
		Case "k"  ; Return "%6B"
		Case "l"  ; Return "%6C"
		Case "m"  ; Return "%6D"
		Case "n"  ; Return "%6E"
		Case "o"  ; Return "%6F"
		Case "p"  ; Return "%70"
		Case "q"  ; Return "%71"
		Case "r"  ; Return "%72"
		Case "s"  ; Return "%73"
		Case "t"  ; Return "%74"
		Case "u"  ; Return "%75"
		Case "v"  ; Return "%76"
		Case "w"  ; Return "%77"
		Case "x"  ; Return "%78"
		Case "y"  ; Return "%79"
		Case "z"  ; Return "%7A"
		End Rem
		Case "{"  ; Return "%7B"
		Case "|"  ; Return "%7C"
		Case "}"  ; Return "%7D"
		Case "~~" ; Return "%7E"
		Case ""  ; Return "%A2"
		Case ""  ; Return "%A3"
		Case ""  ; Return "%A5"
		Case "|"  ; Return "%A6"
		Case ""  ; Return "%A7"
		Case ""  ; Return "%AB"
		Case ""  ; Return "%AC"
		Case ""  ; Return "%AD"
		Case ""  ; Return "%B0"
		Case ""  ; Return "%B1"
		Case ""  ; Return "%B2"
		Case ","  ; Return "%B4"
		Case ""  ; Return "%B5"
		Case ""  ; Return "%BB"
		Case ""  ; Return "%BC"
		Case ""  ; Return "%BD"
		Case ""  ; Return "%BF"
		Case ""  ; Return "%C0"
		Case ""  ; Return "%C1"
		Case ""  ; Return "%C2"
		Case ""  ; Return "%C3"
		Case ""  ; Return "%C4"
		Case ""  ; Return "%C5"
		Case ""  ; Return "%C6"
		Case ""  ; Return "%C7"
		Case ""  ; Return "%C8"
		Case ""  ; Return "%C9"
		Case ""  ; Return "%CA"
		Case ""  ; Return "%CB"
		Case ""  ; Return "%CC"
		Case ""  ; Return "%CD"
		Case ""  ; Return "%CE"
		Case ""  ; Return "%CF"
		Case ""  ; Return "%D0"
		Case ""  ; Return "%D1"
		Case ""  ; Return "%D2"
		Case ""  ; Return "%D3"
		Case ""  ; Return "%D4"
		Case ""  ; Return "%D5"
		Case ""  ; Return "%D6"
		Case ""  ; Return "%D8"
		Case ""  ; Return "%D9"
		Case ""  ; Return "%DA"
		Case ""  ; Return "%DB"
		Case ""  ; Return "%DC"
		Case ""  ; Return "%DD"
		Case ""  ; Return "%DE"
		Case ""  ; Return "%DF"
		Case ""  ; Return "%E0"
		Case ""  ; Return "%E1"
		Case ""  ; Return "%E2"
		Case ""  ; Return "%E3"
		Case ""  ; Return "%E4"
		Case ""  ; Return "%E5"
		Case ""  ; Return "%E6"
		Case ""  ; Return "%E7"
		Case ""  ; Return "%E8"
		Case ""  ; Return "%E9"
		Case ""  ; Return "%EA"
		Case ""  ; Return "%EB"
		Case ""  ; Return "%EC"
		Case ""  ; Return "%ED"
		Case ""  ; Return "%EE"
		Case ""  ; Return "%EF"
		Case ""  ; Return "%F0"
		Case ""  ; Return "%F1"
		Case ""  ; Return "%F2"
		Case ""  ; Return "%F3"
		Case ""  ; Return "%F4"
		Case ""  ; Return "%F5"
		Case ""  ; Return "%F6"
		Case ""  ; Return "%F7"
		Case ""  ; Return "%F8"
		Case ""  ; Return "%F9"
		Case ""  ; Return "%FA"
		Case ""  ; Return "%FB"
		Case ""  ; Return "%FC"
		Case ""  ; Return "%FD"
		Case ""  ; Return "%FE"
		Case ""  ; Return "%FF"
		Default   ;
			If Asc(Char) = 8 Then
				Return "%08"
			Else
				Return Char
			EndIf
	End Select

End Function

